//*----------------------------------------------------------------------------
/*
 * light weight WS2812 lib
 *
 * Created: 07.04.2013 15:57:49 - v0.1
 *			21.04.2013 15:57:49 - v0.2 - Added 12 Mhz code, cleanup
 *			07.05.2013          - v0.4 - size optimization, disable irq
 *			20.05.2013          - v0.5 - Fixed timing bug from size optimization
 *			27.05.2013			- v0.6 - Major update: Changed I/O Port access to byte writes
 *										 instead of bit manipulation. This removes this timing
 *										 discrepancy between standard AVR and reduced core so that
 *										 only one routine is required. This comes at the cost of
 *										 additional register usage.
 *			28.05.2013			- v0.7 - Optimized timing and size of 8 and 12 Mhz routines. 
 *										 All routines are within datasheet specs now, except of
 *										 9.6 Mhz which is marginally off.			
 *			03.06.2013			- v0.8 - 9.6 Mhz implementation now within specifications.
 *								-		 brvs->brcs. Loops terminate correctly
 *  Author: Tim (cpldcpu@gmail.com) 
 */ 
//*----------------------------------------------------------------------------
// Tested:
// Attiny 85	4 MHz, 8 MHz, 16 MHz, 16.5 MHz (Little-Wire)
// Attiny 13A	9.6 MHz
// Attiny 10	4 Mhz, 8 Mhz (Reduced core)
// Atmega 8		12 Mhz
//*----------------------------------------------------------------------------
#include "light_ws2812.h"
//*----------------------------------------------------------------------------
/*
	Timing optimized for 8Mhz AVR 

	The total length of each bit is 1.25s (10 cycles @ 8Mhz)
	* At 0s the dataline is pulled high.  (cycle 1+0=1)
	* To send a zero the dataline is pulled low after 0.375s (1+3=4 cycles).
	* To send a one the dataline is pulled low after 0.625s (1+5=6 cycles).

	Final timing: 
	* 10 cycles for bits 7-1
	* 14 cycles for bit 0		
	
	- The bit 0 timing exceeds the 1.25s bit-timing by 500s, which is still
	  within datasheet tolerances (600s)
*/
//*----------------------------------------------------------------------------
void ws2812_sendarray_mask(uint8_t *data, uint16_t datlen, uint8_t maskhi)
{
	uint8_t curbyte = 0, ctr, masklo = ~maskhi;
	
	asm volatile(
	"		in		%0,%2		\n\t"		
	"		or		%3,%0		\n\t"
	"		and		%4,%0		\n\t"	
	"olop%=:subi	%A6,1		\n\t"		// 9
	"		sbci	%B6,0		\n\t"		// 10
	"		brcs	exit%=		\n\t"		// 11
	"		ld		%1,X+		\n\t"		// 13
	"		ldi		%0,8		\n\t"		// 14
		
	"loop%=:out		%2,	%3		\n\t"		// 1
	"		lsl		%1			\n\t"		// 2
		
	"		brcs	.+2			\n\t"		// 3nt / 4t
	"		out		%2,	%4		\n\t"		// 4
	"		dec		%0			\n\t"		// 5
	"		out		%2,	%4		\n\t"		// 6
	"		breq	olop%=		\n\t"		// 7nt  / 8t
	"		nop					\n\t"		// 8
	"		rjmp	loop%=		\n\t"		// 10 
	"exit%=:					\n\t"		// 
	:	"=&d" (ctr)
	:	"r" (curbyte), "I" (_SFR_IO_ADDR(ws2812_port)), "r" (maskhi), "r" (masklo), "x" (data), "r" (datlen)
	);
}
//*----------------------------------------------------------------------------
